local AceGUI = LibStub("AceGUI-3.0")

local scrollBench = nil;
local benchPlayer = nil;
local editPlayer;

-- Bench toggle action
-- =========================
function MorthalinRaidTools:ToggleBench()
	if MorthalinRaidTools.ui.benchFrame:IsVisible() then
		MorthalinRaidTools.ui.benchFrame:Hide();
	else
		statusGroup:ReleaseChildren();
		if self.db.global.acceptBench then
			local btnBenchInv = AceGUI:Create("Button");
			btnBenchInv:SetText("Disable invites on bench");
			btnBenchInv:SetFullWidth(true);
			btnBenchInv:SetCallback("OnClick", function() self:ToggleInvites() end);
			statusGroup:AddChild(btnBenchInv);
			self.ui.benchFrame:SetStatusText("Addon accepting raid members");
		else
			local btnActivateBench = AceGUI:Create("Button");
			btnActivateBench:SetText("Allow bench invites");
			btnActivateBench:SetFullWidth(true);
			btnActivateBench:SetCallback("OnClick", function() self:ToggleInvites(); end);
			statusGroup:AddChild(btnActivateBench);
			self.ui.benchFrame:SetStatusText("");
		end	
		MorthalinRaidTools.ui.benchFrame:Show();
		MorthalinRaidTools:UpdateBenchView();
	end
end

function MorthalinRaidTools:ConstructBenchFrame()
	self.ui.benchFrame = AceGUI:Create("Frame");
	self.ui.benchFrame:SetTitle("Bench");
	self.ui.benchFrame:SetStatusText("");
	self.ui.benchFrame:SetLayout("Flow");
	self.ui.benchFrame:SetWidth(430);
	self.ui.benchFrame:EnableResize(false);
	self.ui.benchFrame.frame:EnableMouse(true);
	self.ui.benchFrame.frame:SetUserPlaced(true);
	self.ui.benchFrame:SetCallback("OnClose", function(widget)
		if(self:WhispererVisible()) then
			self:HideWhisperer();
		end
	end);
	
	-- TABLE GROUP
	local tableGroup = AceGUI:Create("SimpleGroup");
	tableGroup:SetWidth(200);
	tableGroup:SetLayout("Flow");
	self.ui.benchFrame:AddChild(tableGroup);
	
	-- TABLE HEADER
	local tableHeader = AceGUI:Create("SimpleGroup");
	tableHeader:SetFullWidth(true);
	tableHeader:SetLayout("Flow");
	tableGroup:AddChild(tableHeader);

	local btn = AceGUI:Create("InteractiveLabel");
	btn:SetWidth(150);
	btn:SetText("Players");
	tableHeader:AddChild(btn);
	
	--TABLE DATA PART
	scrollcontainer = AceGUI:Create("SimpleGroup");
	scrollcontainer:SetFullWidth(true);
	scrollcontainer:SetHeight(380);
	scrollcontainer:SetLayout("Fill");
	tableGroup:AddChild(scrollcontainer);
	scrollcontainer:ClearAllPoints();
	scrollcontainer.frame:SetPoint("BOTTOM", 0, 20);

	scrollBench = CreateFrame("ScrollFrame", nil, scrollcontainer.frame, "myScrollFrame");
	HybridScrollFrame_CreateButtons(scrollBench, "benchListItemTemplate");
	HybridScrollFrame_SetDoNotHideScrollBar(scrollBench, true);
	scrollBench.update = function() 
		self:UpdateBenchView() 
	end
	
	local lable = AceGUI:Create("Label");
	lable:SetFullWidth(true);
	lable:SetText("For invite click left");
	tableGroup:AddChild(lable);
	
	lable = AceGUI:Create("Label");
	lable:SetFullWidth(true);
	lable:SetText("For remove click right");
	tableGroup:AddChild(lable);
	
	-- CONTROLS GROUP
	local controlsGroup = AceGUI:Create("SimpleGroup");
	controlsGroup:SetWidth(190);
	controlsGroup:SetLayout("Flow");
	self.ui.benchFrame:AddChild(controlsGroup);
	controlsGroup:ClearAllPoints();
	controlsGroup:SetPoint("TOPRIGHT", 0, -10);
	
	statusGroup = AceGUI:Create("SimpleGroup");
	statusGroup:SetFullWidth(true);
	statusGroup:SetLayout("Flow");
	controlsGroup:AddChild(statusGroup);
	
	if self.db.global.acceptBench then
		local btnBenchInv = AceGUI:Create("Button");
		btnBenchInv:SetText("Disable invites on bench");
		btnBenchInv:SetFullWidth(true);
		btnBenchInv:SetCallback("OnClick", function() self:ToggleInvites() end);
		statusGroup:AddChild(btnBenchInv);
		self.ui.benchFrame:SetStatusText("Addon accepting raid members");
	else
		local btnActivateBench = AceGUI:Create("Button");
		btnActivateBench:SetText("Allow bench invites");
		btnActivateBench:SetFullWidth(true);
		btnActivateBench:SetCallback("OnClick", function() self:ToggleInvites(); end);
		statusGroup:AddChild(btnActivateBench);
		self.ui.benchFrame:SetStatusText("");
	end
	
	local btnEmptyBench = AceGUI:Create("Button");
	btnEmptyBench:SetText("Empty bench");
	btnEmptyBench:SetFullWidth(true);
	btnEmptyBench:SetCallback("OnClick", function() self:ResetList(); end)
	controlsGroup:AddChild(btnEmptyBench);
	
	editPlayer = AceGUI:Create("EditBox");
	editPlayer:SetLabel("Insert player:");
	editPlayer:DisableButton(true);
	editPlayer:SetCallback("OnTextChanged", function(widget, event, text)
		benchPlayer = text;
		if(string.len(text) ~= 0) then
			self:ConstructWhisperer(editPlayer, benchPlayer, 1);
		else
			self:HideWhisperer();
		end
	end)
	editPlayer:SetCallback("OnEnterPressed", function(widget, event, text) self:AddOnBench(text, text); editPlayer:SetText(""); end)
	controlsGroup:AddChild(editPlayer);
	
	local btnAddBench = AceGUI:Create("Button");
	btnAddBench:SetText("Add Player");
	btnAddBench:SetFullWidth(true);
	btnAddBench:SetCallback("OnClick", function() self:AddOnBench(benchPlayer, benchPlayer); editPlayer:SetText(""); end)
	controlsGroup:AddChild(btnAddBench);
	
	local btnBroadcastBench = AceGUI:Create("Button");
	btnBroadcastBench:SetText("Broadcast bench");
	btnBroadcastBench:SetFullWidth(true);
	btnBroadcastBench:SetCallback("OnClick", function() self:BroadcastBench(); end)
	controlsGroup:AddChild(btnBroadcastBench);
	
	local btnAskForBroadcast = AceGUI:Create("Button");
	btnAskForBroadcast:SetText("Request bench broadcast");
	btnAskForBroadcast:SetFullWidth(true);
	btnAskForBroadcast:SetCallback("OnClick", function() self:AskForBenchBroadcast(); end)
	controlsGroup:AddChild(btnAskForBroadcast);
	
	self.ui.benchFrame:Hide();
end

function MorthalinRaidTools:UpdateBenchView()
	if (scrollBench == nil) then
		return;
	end
	local buttons = HybridScrollFrame_GetButtons(scrollBench);
	local offset = HybridScrollFrame_GetOffset(scrollBench);
	local rows = self.db.global.bench.players;
	for buttonIndex = 1, #buttons do
		local button = buttons[buttonIndex];
		local itemIndex = buttonIndex + offset;
		
		if (itemIndex <= #rows) then
			button:SetID(itemIndex);
			button.Player:SetText("|cff" .. rows[itemIndex].color .. rows[itemIndex].name);
			button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
			button:SetScript('OnClick', function(obj, btn, down)
				if(btn == "RightButton") then
					MorthalinRaidTools:RemoveFromBench(rows[itemIndex].name);
				else
					InviteUnit(rows[itemIndex].name);
				end
			end);
			button:Show();
		else
			button:Hide();
		end
	end

	local buttonHeight = scrollBench.buttonHeight;
	local totalHeight = #rows * buttonHeight;
	local shownHeight = #buttons * buttonHeight;

	HybridScrollFrame_Update(scrollBench, totalHeight, shownHeight);
end

function MorthalinRaidTools:ToggleInvites()
	statusGroup:ReleaseChildren();
	
	if self.db.global.acceptBench then
		local btnActivateBench = AceGUI:Create("Button");
		btnActivateBench:SetText("Allow bench invites");
		btnActivateBench:SetFullWidth(true);
		btnActivateBench:SetCallback("OnClick", function() self:ToggleInvites(); end);
		statusGroup:AddChild(btnActivateBench);
		self.ui.benchFrame:SetStatusText("");
	else
		local btnBenchInv = AceGUI:Create("Button");
		btnBenchInv:SetText("Disable invites on bench");
		btnBenchInv:SetFullWidth(true);
		btnBenchInv:SetCallback("OnClick", function() self:ToggleInvites() end);
		statusGroup:AddChild(btnBenchInv);
		self.ui.benchFrame:SetStatusText("Addon accepting raid members");
	end
	
	self:AllowChat();
end

function MorthalinRaidTools:ShowBench(obj)
	GameTooltip:SetOwner(obj,"ANCHOR_LEFT");
	if(#self.db.global.bench.players ~= 0) then
		GameTooltip:AddLine("Players on bench:");
		for index, player in pairs(self.db.global.bench.players) do
			GameTooltip:AddLine("|cff" .. player.color .. player.name);
		end
	else
		GameTooltip:AddLine("|cffff0000List is empty!");
	end
	GameTooltip:Show();
end

function MorthalinRaidTools:WhispererAddOnBench(text)
	local correction = string.sub(text, 11, -1);
	self:AddOnBench(correction, correction);
	benchPlayer = "";
	editPlayer:SetText("");
	self:HideWhisperer();
end