local AceGUI = LibStub("AceGUI-3.0")

local scrollScheduler = nil;
local raidPick = nil;
MorthalinRaidTools.autoInvSchedule = false;
local autoInviteIndex = 0;
local pickedPlayerIndex = 0;
local statusGroup = nil;

-- Bench toggle action
-- =========================
function MorthalinRaidTools:ToggleScheduler()
	if MorthalinRaidTools.ui.schedulerFrame:IsVisible() then
		MorthalinRaidTools.ui.schedulerFrame:Hide();
	else
		MorthalinRaidTools.ui.schedulerFrame:Show();
		MorthalinRaidTools:LoadScheduleRaids();
	end
end

function MorthalinRaidTools:ConstructSchedulerFrame()
	self.ui.schedulerFrame = AceGUI:Create("Frame");
	self.ui.schedulerFrame:SetTitle("Scheduler");
	self.ui.schedulerFrame:SetStatusText("");
	self.ui.schedulerFrame:SetLayout("Flow");
	self.ui.schedulerFrame:SetWidth(430);
	self.ui.schedulerFrame:EnableResize(false);
	self.ui.schedulerFrame.frame:EnableMouse(true);
	self.ui.schedulerFrame.frame:SetUserPlaced(true);
	self.ui.schedulerFrame:SetCallback("OnClose", function(widget) end);
	
	local raidPickGroup = AceGUI:Create("SimpleGroup");
	raidPickGroup:SetFullWidth();
	raidPickGroup:SetLayout("Flow");
	self.ui.schedulerFrame:AddChild(raidPickGroup);
	
	raidPick = AceGUI:Create("Dropdown");
	raidPick:SetLabel("Players");
	raidPick:SetWidth(200);
	raidPick:SetCallback("OnValueChanged", function(obj, func, value) MorthalinRaidTools:UpdateSchedulerView(); end);
	raidPick:SetCallback("OnEnter", function()
		GameTooltip:SetOwner(raidPick.frame,"ANCHOR_RIGHT");
		GameTooltip:AddLine(raidPick.list[raidPick:GetValue()]);
		GameTooltip:Show();
	end);
	raidPick:SetCallback("OnLeave", function() self:HideTooltip() end);
	raidPickGroup:AddChild(raidPick);
	
	-- TABLE GROUP
	local tableGroup = AceGUI:Create("SimpleGroup");
	tableGroup:SetWidth(200);
	tableGroup:SetLayout("Flow");
	self.ui.schedulerFrame:AddChild(tableGroup);
	
	-- TABLE HEADER
	local tableHeader = AceGUI:Create("SimpleGroup");
	tableHeader:SetFullWidth(true);
	tableHeader:SetLayout("Flow");
	tableGroup:AddChild(tableHeader);

	local btn = AceGUI:Create("InteractiveLabel");
	btn:SetWidth(150);
	btn:SetText("Players");
	tableHeader:AddChild(btn);
	
	--TABLE DATA PART
	scrollcontainer = AceGUI:Create("SimpleGroup");
	scrollcontainer:SetFullWidth(true);
	scrollcontainer:SetHeight(350);
	scrollcontainer:SetLayout("Fill");
	tableGroup:AddChild(scrollcontainer);
	scrollcontainer:ClearAllPoints();
	scrollcontainer.frame:SetPoint("BOTTOM", 0, 20);

	scrollScheduler = CreateFrame("ScrollFrame", nil, scrollcontainer.frame, "myScrollFrame");
	HybridScrollFrame_CreateButtons(scrollScheduler, "benchListItemTemplate");
	HybridScrollFrame_SetDoNotHideScrollBar(scrollScheduler, true);
	scrollScheduler.update = function() 
		self:UpdateSchedulerView() 
	end
	
	local lable = AceGUI:Create("Label");
	lable:SetFullWidth(true);
	lable:SetText("For invite click left | For remove click middle | For replace click right");
	self.ui.schedulerFrame:AddChild(lable);
	
	-- CONTROLS GROUP
	local controlsGroup = AceGUI:Create("SimpleGroup");
	controlsGroup:SetWidth(190);
	controlsGroup:SetLayout("Flow");
	self.ui.schedulerFrame:AddChild(controlsGroup);
	controlsGroup:ClearAllPoints();
	controlsGroup:SetPoint("TOPRIGHT", 0, -50);
	
	local btnImport = AceGUI:Create("Button");
	btnImport:SetText("Import new raid");
	btnImport:SetFullWidth(true);
	btnImport:SetCallback("OnClick", function() MorthalinRaidTools:ShowEditWindow(true, 1); end)
	controlsGroup:AddChild(btnImport);
	
	local btnCheck = AceGUI:Create("Button");
	btnCheck:SetText("Check Players in raid");
	btnCheck:SetFullWidth(true);
	btnCheck:SetCallback("OnClick", function() MorthalinRaidTools:CheckNotInRaid(); MorthalinRaidTools:CheckInRaidNotInList(); end)
	controlsGroup:AddChild(btnCheck);
	
	local btnRemove = AceGUI:Create("Button");
	btnRemove:SetText("Remove selected raid");
	btnRemove:SetFullWidth(true);
	btnRemove:SetCallback("OnClick", function() MorthalinRaidTools:RemoveScheduledRaid(); end)
	controlsGroup:AddChild(btnRemove);
	
	local btnAddPlayer = AceGUI:Create("Button");
	btnAddPlayer:SetText("Add player to raid");
	btnAddPlayer:SetFullWidth(true);
	btnAddPlayer:SetCallback("OnClick", function() MorthalinRaidTools:ConstructTextConfirm("Player add", "Pick character you want to add to scheduled raid.", "Player add", 2); end)
	controlsGroup:AddChild(btnAddPlayer);
	
	statusGroup = AceGUI:Create("SimpleGroup");
	statusGroup:SetFullWidth(true);
	statusGroup:SetLayout("Flow");
	controlsGroup:AddChild(statusGroup);
	
	if (MorthalinRaidTools.autoInvSchedule) then
		local btnBenchInv = AceGUI:Create("Button");
		btnBenchInv:SetText("Disable automatic invites");
		btnBenchInv:SetFullWidth(true);
		btnBenchInv:SetCallback("OnClick", function() MorthalinRaidTools:ToggleInvitesSchedule(); end);
		statusGroup:AddChild(btnBenchInv);
		self.ui.schedulerFrame:SetStatusText("Addon accepting raid members");
	else
		local btnActivateBench = AceGUI:Create("Button");
		btnActivateBench:SetText("Allow automatic invites");
		btnActivateBench:SetFullWidth(true);
		btnActivateBench:SetCallback("OnClick", function() MorthalinRaidTools:ToggleInvitesSchedule(); end);
		statusGroup:AddChild(btnActivateBench);
		self.ui.schedulerFrame:SetStatusText("");
	end
	
	self.ui.schedulerFrame:Hide();
end

function MorthalinRaidTools:UpdateSchedulerView()
	if (scrollScheduler == nil) then
		return;
	end
	local buttons = HybridScrollFrame_GetButtons(scrollScheduler);
	local offset = HybridScrollFrame_GetOffset(scrollScheduler);
	local rows = raidPick:GetValue() == nil and {} or MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members;
	for buttonIndex = 1, #buttons do
		local button = buttons[buttonIndex];
		local itemIndex = buttonIndex + offset;
		
		if (itemIndex <= #rows) then
			button:SetID(itemIndex);
			button.Player:SetText("|cff" .. rows[itemIndex].color .. rows[itemIndex].name);
			button:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp");
			button:SetScript('OnClick', function(obj, btn, down)
				if(btn == "RightButton") then
					pickedPlayerIndex = itemIndex;
					MorthalinRaidTools:ConstructTextConfirm("Player change", "Pick character to replace |cff" .. rows[itemIndex].color .. rows[itemIndex].name .. "|r from scheduled raid.", "Player change", 1);
				elseif(btn == "MiddleButton") then
					MorthalinRaidTools:RemovePlayerFromSchedule(itemIndex);
				else
					InviteUnit(rows[itemIndex].name);
				end
			end);
			button:Show();
		else
			button:Hide();
		end
	end

	local buttonHeight = scrollScheduler.buttonHeight;
	local totalHeight = #rows * buttonHeight;
	local shownHeight = #buttons * buttonHeight;

	HybridScrollFrame_Update(scrollScheduler, totalHeight, shownHeight);
end

function MorthalinRaidTools:LoadScheduleRaids()
	raidPick:SetValue(nil);
	raidPick:SetList({});
	MorthalinRaidTools:RemoveExpiredSchedule();
	local function sortRaids(a, b)
		return a.raidDate < b.raidDate;
	end
	table.sort(MorthalinRaidTools.db.global.schedule, sortRaids);
	for i = 1, #MorthalinRaidTools.db.global.schedule do
		raidPick:AddItem(i, MorthalinRaidTools.db.global.schedule[i].name .. " - " .. date("%d.%m.%y %H:%M", MorthalinRaidTools.db.global.schedule[i].raidDate));
	end
	
	if(#MorthalinRaidTools.db.global.schedule ~= 0) then
		raidPick:SetValue(1);
	end
	MorthalinRaidTools:UpdateSchedulerView();
end

function MorthalinRaidTools:RemoveScheduledRaid()
	table.remove(MorthalinRaidTools.db.global.schedule, raidPick:GetValue());
	MorthalinRaidTools:LoadScheduleRaids();
end

function MorthalinRaidTools:CheckNotInRaid()
	MorthalinRaidTools:Print("Missing players:");
	for i = 1, #MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members do
		local found = false;
		for j = 1, 40 do
			local name = GetRaidRosterInfo(j);
			if(name ~= nil and name == MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members[i].name) then
				found = true;
			end
		end
		if(found == false) then
			MorthalinRaidTools:Print("|cff" .. MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members[i].color .. MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members[i].name);
		end
	end
end

function MorthalinRaidTools:CheckInRaidNotInList()
	MorthalinRaidTools:Print("|cffff0000Players not in list:");
	for j = 1, 40 do
		local name, rank, subgroup, level, class = GetRaidRosterInfo(j);
		local found = false;
		for i = 1, #MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members do
			if(name ~= nil and name == MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members[i].name) then
				found = true;
			end
		end
		if(found == false and name ~= nil) then
			MorthalinRaidTools:Print("|cff" .. MorthalinRaidTools:GetClassColor(class) .. Ambiguate(name, "short"));
		end
	end
end

function MorthalinRaidTools:ToggleInvitesSchedule()
	statusGroup:ReleaseChildren();
	
	if MorthalinRaidTools.autoInvSchedule then
		local btnActivateBench = AceGUI:Create("Button");
		btnActivateBench:SetText("Allow automatic invites");
		btnActivateBench:SetFullWidth(true);
		btnActivateBench:SetCallback("OnClick", function() self:ToggleInvitesSchedule(); end);
		statusGroup:AddChild(btnActivateBench);
		self.ui.schedulerFrame:SetStatusText("");
		MorthalinRaidTools.autoInvSchedule = false;
	else
		local btnBenchInv = AceGUI:Create("Button");
		btnBenchInv:SetText("Disable automatic invites");
		btnBenchInv:SetFullWidth(true);
		btnBenchInv:SetCallback("OnClick", function() self:ToggleInvitesSchedule() end);
		statusGroup:AddChild(btnBenchInv);
		self.ui.schedulerFrame:SetStatusText("Addon accepting raid members");
		MorthalinRaidTools.autoInvSchedule = true;
	end
	MorthalinRaidTools:ToggleAutoInvites();
end

function MorthalinRaidTools:ToggleAutoInvites()
	autoInviteIndex = raidPick:GetValue();
	local index = 1;
	if(MorthalinRaidTools.autoInvSchedule) then
		if(self.db.global.acceptBench == false) then
			MorthalinRaidTools:RegisterEvent("CHAT_MSG_GUILD", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
			MorthalinRaidTools:RegisterEvent("CHAT_MSG_WHISPER", function(...) MorthalinRaidTools:ChatChannelCallback(...) end);
		end
		for i = 1, #MorthalinRaidTools.db.global.schedule[autoInviteIndex].members do
			local playerInfo = MorthalinRaidTools:GetPlayerInfo(MorthalinRaidTools.db.global.schedule[autoInviteIndex].members[i].name);
			if(MorthalinRaidTools:IsInRaid(MorthalinRaidTools.db.global.schedule[autoInviteIndex].members[i].name) == false and playerInfo.online) then
				MorthalinRaidTools:ScheduleTimer(function() InviteUnit(MorthalinRaidTools.db.global.schedule[autoInviteIndex].members[i].name); end, math.ceil(index/5));
				index = index + 1;
			end
		end
	else
		if(self.db.global.acceptBench == false) then
			MorthalinRaidTools:UnregisterEvent("CHAT_MSG_GUILD");
			MorthalinRaidTools:UnregisterEvent("CHAT_MSG_WHISPER");
		end
		autoInviteIndex = 0;
	end
end

function MorthalinRaidTools:CheckLoggedPlayer(name, fromChat)
	local found = false;
	for i = 1, #MorthalinRaidTools.db.global.schedule[autoInviteIndex].members do
		if(MorthalinRaidTools.db.global.schedule[autoInviteIndex].members[i].name == name) then
			if(MorthalinRaidTools:IsInRaid(name) == false) then
				MorthalinRaidTools:ScheduleTimer(function() InviteUnit(name); end, 1);
				found = true;
			end
			i = #MorthalinRaidTools.db.global.schedule[autoInviteIndex].members;
		end
	end
	if(fromChat ~= nil and found == false) then
		SendChatMessage("You are on bench or not assigned in actual raid.", "WHISPER", "Common", name);
	end
end

function MorthalinRaidTools:ReplacePlayer(newPlayer)
	local temp;
	print(newPlayer);
	if(string.len(newPlayer) > 1) then
		temp = string.upper(string.utf8sub(newPlayer, 1, 1))..string.lower(string.utf8sub(newPlayer, 2));
	elseif(string.len(newPlayer) == 1) then
		temp = string.upper(string.utf8sub(newPlayer, 1, 1));
	end
	self:HideWhisperer();
	if(self.ui.confirmTextFrame ~= nil and self.ui.confirmTextFrame:IsVisible()) then
		self.ui.confirmTextFrame:Hide();
	end
	if(MorthalinRaidTools:IsInScheduledRaid(temp)) then
		MorthalinRaidTools:Print("|cffff0000Player is already in scheduled raid!");
	else
		local playerInfo = MorthalinRaidTools:GetPlayerInfo(temp);
		if(playerInfo ~= nil) then
			MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members[pickedPlayerIndex].name = temp;
			MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members[pickedPlayerIndex].class = playerInfo.class;
			MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members[pickedPlayerIndex].color = MorthalinRaidTools:GetClassColor(playerInfo.class);
			
			MorthalinRaidTools:UpdateSchedulerView();
		else
			MorthalinRaidTools:Print("|cffff0000Player does not exist or is not in guild!");
		end
	end
	pickedPlayerIndex = 0;
end

function MorthalinRaidTools:IsInScheduledRaid(player)
	for i = 1, #MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members do
		if(MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members[i].name == player) then
			return true;
		end
	end
	return false;
end

function MorthalinRaidTools:RemovePlayerFromSchedule(index)
	table.remove(MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members, index);
	MorthalinRaidTools:UpdateSchedulerView();
end

function MorthalinRaidTools:AddPlayerSchedule(newPlayer)
	local temp;
	if(string.len(newPlayer) > 1) then
		temp = string.upper(string.utf8sub(newPlayer, 1, 1))..string.lower(string.utf8sub(newPlayer, 2));
	elseif(string.len(newPlayer) == 1) then
		temp = string.upper(string.utf8sub(newPlayer, 1, 1));
	end
	self:HideWhisperer();
	if(self.ui.confirmTextFrame ~= nil and self.ui.confirmTextFrame:IsVisible()) then
		self.ui.confirmTextFrame:Hide();
	end
	if(MorthalinRaidTools:IsInScheduledRaid(temp)) then
		MorthalinRaidTools:Print("|cffff0000Player is already in scheduled raid!");
	else
		local playerInfo = MorthalinRaidTools:GetPlayerInfo(temp);
		if(playerInfo ~= nil) then
			table.insert(MorthalinRaidTools.db.global.schedule[raidPick:GetValue()].members, {name = temp, class = playerInfo.class, color = MorthalinRaidTools:GetClassColor(playerInfo.class)});
			MorthalinRaidTools:UpdateSchedulerView();
		else
			MorthalinRaidTools:Print("|cffff0000Player does not exist or is not in guild!");
		end
	end
end

function MorthalinRaidTools:IsInRaid(player)
	for j = 1, 40 do
		local name = GetRaidRosterInfo(j);
		if(name ~= nil and name == player) then
			return true;
		end
	end
	return false;
end
